names = ['Иван', 'Мария', 'Петр', 'Илья']
names_1 = []
names_summ = []
first_letter_list = []


def keys_teshaurus(names):
    names.sort()
    for i in names:
        first_letter = (names[names.index(i)])[0]
        if names.index(i) == len(names) - 1:
            first_letter_list.append(first_letter)
            names_1.append(names[names.index(i)])
            if (names[names.index(i)])[0] == (names[names.index(i) - 1])[0]:
                first_letter_list.pop()
                names_1.pop()
            break
        else:
            if (names[names.index(i)])[0] == (names[names.index(i) + 1])[0]:
                first_letter_repeat = names[names.index(i)][0]
                first_letter_list.append(first_letter_repeat)
                names_summ = list(filter(lambda el: el.startswith(first_letter_repeat), names))
                names_1.append(names_summ)
            elif (names[names.index(i)])[0] == (names[names.index(i) - 1])[0]:
                True
            else:
                first_letter_list.append(first_letter)
                names_1.append(names[names.index(i)])
    teshaurus = dict(zip(first_letter_list, names_1))
    print(teshaurus)
    return teshaurus


keys_teshaurus(names)



"""Утилиты"""
import sys
import json
import logging
import traceback

from common.variables import MAX_PACKAGE_LENGTH, ENCODING

"""Простейший декоратор-класс"""


class Log:
    """Класс-декоратор"""
    def __call__(self, func):
        def decorated(*args, **kwargs):
            """Обертка"""
            res = func(*args, **kwargs)
            script = sys.argv[0].split('/')[-1]
            function_call = repr(traceback.extract_stack()[1]).split(' ')[-1].strip('>')
            if script == 'client.py':
                # traceback.print_stack()
                # print(function_call)
                logger = logging.getLogger('client')
                logger.info(f'Функция {func.__name__} вызвана из функции {function_call}')
                logger.debug(f'client: {func.__name__}({args}, {kwargs})')
                # print(f'client: {func.__name__}({args}, {kwargs}) = {res}')
            elif script == 'server_PyQT.py':
                # print(function_call)
                logger = logging.getLogger('server')
                logger.debug(f'server: {func.__name__}({args}, {kwargs})')
                logger.info(f'Функция {func.__name__} вызвана из функции {function_call}')
            return res
        return decorated


@Log()
def get_message(client):
    """
    Утилита приёма и декодирования сообщения.
    Принимает байты, выдаёт словарь, если принято что-то
    другое возвращает ValueError (ошибку значения)
    """

    encoded_response = client.recv(MAX_PACKAGE_LENGTH)
    if isinstance(encoded_response, bytes):
        json_response = encoded_response.decode(ENCODING)
        if isinstance(json_response, str):
            response = json.loads(json_response)
            if isinstance(response, dict):
                return response
            raise ValueError
        raise ValueError
    raise ValueError


@Log()
def send_message(sock, message):
    """
    Утилита кодирования и отправки сообщения:
    принимает для отправки словарь, получает из него строку,
    далее превращает строку в байты и отправляет.
    """
    if not isinstance(message, dict):
        raise TypeError
    js_message = json.dumps(message)
    encoded_message = js_message.encode(ENCODING)

    sock.send(encoded_message)



"""Программа Сервер. Используется идея:
При регистрации на сервер, Имя Пользователя И адрес его Сокета записываются в словарь, и добавляются в
глобальный список USERS. Отправка сообщения происходит при совпадении адреса Сокета и текущего Клиента"""
import binascii
import datetime
import hmac
import os
import socket
import threading
from PyQt6.QtCore import pyqtSignal, QObject
from select import select
from common.descriptor import NonNegative
from server.my_socket import MySocket
from common.utils import get_message, send_message
from logs.server_log_config import log
from common.variables import *

logger = log
users = []


class Server(threading.Thread, QObject):
    port = NonNegative()
    new_connection = pyqtSignal()

    def __init__(self, listen_address, port, database):
        # Параметры подключения
        self.addr = listen_address
        self.port = port

        # База данных сервера
        self.database = database
        # Список подключенных клиентов
        self.clients = []
        self.clients_socket_names = []
        # Список сообщений
        self.messages = []
        threading.Thread.__init__(self)
        QObject.__init__(self)

    def process_client_message(self, message, client):
        global users
        # print(message)
        logger.debug(f'Получено сообщение от клиента {message}')
        if ACTION in message and message[ACTION] == PRESENCE and TIME in message \
                and USER in message and message[USER][ACCOUNT_NAME]:
            res_auth = self.autorize_user(message, client)
            if res_auth != RESPONSE_400:
                username = message[USER][ACCOUNT_NAME]
                ip_address = message['user']['sock'][0]
                port = message['user']['sock'][1]
                self.database.user_login(username, ip_address, port)
                self.new_connection.emit()
                users.append(message[USER])
                # self.service_update_lists()
                return {
                    RESPONSE: 200,
                    'data': None,
                    'login': message['user']['account_name'],
                    'sock': message['user']['sock']
                }
        elif ACTION in message and message[ACTION] == 'get_contacts' and TIME in message \
                and USER in message and message[USER][ACCOUNT_NAME]:
            username = message[USER][ACCOUNT_NAME]
            alert = []
            res = sorted(self.database.contacts_list(username))
            for item in res:
                if item.contact_name not in alert:
                    alert.append(item.contact_name)
            # print(alert)
            logger.info(f'Cформирован список контактов клиента {username}')

            return {
                RESPONSE: 202,
                'alert': alert,
                'login': message['user']['account_name'],
                'sock': message['user']['sock']
            }
        elif ACTION in message and message[ACTION] == 'add_contact' and TIME in message \
                and USER in message and message[USER][ACCOUNT_NAME]:
            contact_name = message['contact']
            res_all = sorted(self.database.user_list())
            for item in res_all:
                if item.username == contact_name:
                    return {
                        RESPONSE: 205
                    }

        elif ACTION in message and message[ACTION] == 'del_contact' and TIME in message \
                and USER in message and message[USER][ACCOUNT_NAME]:
            contact_name = message['contact']
            res_all = sorted(self.database.user_list())
            for item in res_all:
                if item.username == contact_name:
                    return {
                        RESPONSE: 210
                    }

        elif ACTION in message and message[ACTION] == 'message' and TIME in message \
                and USER in message and message[USER][ACCOUNT_NAME]:
            try:
                sock_address = [user['sock'] for user in users if user['account_name'] == message['to']][0]
            except IndexError:
                sock_address = ''
            # print(sock_address)
            msg = {
                RESPONSE: 200,
                'data': message['message_text'],
                'login': message['user']['account_name'],
                'to': message['to'],
                'sock_address': sock_address
            }
            # print(msg)
            return msg
        elif ACTION in message and message[ACTION] == 'exit' and TIME in message \
                and USER in message and message[USER][ACCOUNT_NAME]:
            users.remove(message[USER])
            return {RESPONSE: 200,
                    'data': None,
                    'login': message['user']['account_name'],
                    'sock': message['user']['sock']
                    }
        msg = {
            RESPONSE: 400,
            ERROR: 'Bad Request'
        }
        logger.error(f'Bad request 400', msg)
        return {
            RESPONSE: 400,
            ERROR: 'Bad Request'
        }

    def remove_client(self, client):
        """
        Метод обработчик клиента с которым прервана связь.
        Ищет клиента и удаляет его из списков и базы:
        """
        logger.info(f'Клиент {client.getpeername()} отключился от сервера.')
        for user in users:
            if user['account_name'] == client:
                users.remove(user)
                break
        self.clients.remove(client)
        client.close()

    def autorize_user(self, message, sock):
        """Метод реализующий авторизцию пользователей."""
        # Если имя пользователя уже занято то возвращаем 400
        logger.debug(f'Start auth process for {message[USER]}')

        # Проверяем что пользователь зарегистрирован на сервере.
        if not self.database.check_user(message[USER][ACCOUNT_NAME]):
            response = RESPONSE_400
            response[ERROR] = 'Пользователь не зарегистрирован.'
            try:
                logger.debug(f'Unknown username, sending {response}')
                send_message(sock, response)
            except OSError:
                pass
            self.clients.remove(sock)
            sock.close()
        else:
            logger.debug('Correct username, starting passwd check.')
            # Иначе отвечаем 511 и проводим процедуру авторизации
            # Словарь - заготовка
            message_auth = RESPONSE_511
            # Набор байтов в hex представлении
            random_str = binascii.hexlify(os.urandom(64))
            # В словарь байты нельзя, декодируем (json.dumps -> TypeError)
            message_auth[DATA] = random_str.decode('ascii')
            # Создаём хэш пароля и связки с рандомной строкой, сохраняем
            # серверную версию ключа
            hash_ = hmac.new(self.database.get_hash(message[USER][ACCOUNT_NAME]), random_str, 'MD5')
            digest = hash_.digest()
            logger.debug(f'Auth message = {message_auth}')
            try:
                # Обмен с клиентом
                send_message(sock, message_auth)
                ans = get_message(sock)
            except OSError as err:
                logger.debug('Error in auth, data:', exc_info=err)
                sock.close()
                return
            client_digest = binascii.a2b_base64(ans[DATA])
            # Если ответ клиента корректный, то сохраняем его в список
            # пользователей.
            if RESPONSE in ans and ans[RESPONSE] == 511 and hmac.compare_digest(
                    digest, client_digest):
                try:
                    send_message(sock, RESPONSE_200)
                except OSError:
                    self.remove_client(message[USER][ACCOUNT_NAME])

            else:
                response = RESPONSE_400
                response[ERROR] = 'Неверный пароль.'
                try:
                    send_message(sock, response)
                except OSError:
                    pass
                self.clients.remove(sock)
                sock.close()
                return RESPONSE_400

    def service_update_lists(self):
        """Метод реализующий отправки сервисного сообщения 205 клиентам"""
        for client in self.clients:
            try:
                send_message(client, RESPONSE_215)
            except OSError:
                self.clients.remove(client)

    def run(self):
        """
        Загрузка параметров командной строки, если нет параметров, то задаём значения по умолчанию.
        Сначала обрабатываем порт:
        server_PyQT.py -p 8888 -a 127.0.0.1
        """
        logger.info(f'PORT : {self.port} ,IP_ADDRESS {self.addr}')
        s = MySocket(socket.AF_INET, socket.SOCK_STREAM)
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        s.bind((self.addr, self.port))
        s.listen(MAX_CONNECTIONS)
        s.settimeout(2)
        global users
        while True:
            try:
                client, client_address = s.accept()
            except OSError as e:
                print(e.errno)
                pass
            else:
                self.clients.append(client)
                self.clients_socket_names.append(client.getpeername())
                # print(self.clients_socket_names)
            finally:
                recv_data_lst = []
                send_data_lst = []
                # Проверяем на наличие ждущих клиентов
                try:
                    if self.clients:
                        recv_data_lst, send_data_lst, err_lst = select(self.clients, self.clients, [], 0)
                except OSError:
                    pass

                # принимаем сообщения и если ошибка, исключаем клиента.
                if recv_data_lst:
                    for client_with_message in recv_data_lst:
                        try:
                            message_from_client = get_message(client_with_message)

                            print(message_from_client)
                            if message_from_client['action'] == 'exit':
                                recv_data_lst.remove(client_with_message)
                                send_data_lst.remove(client_with_message)
                                self.clients.remove(client_with_message)
                                self.clients_socket_names.remove(client_with_message.getpeername())
                                users.remove(message_from_client[USER])

                            else:
                                self.messages.append(message_from_client)
                                print(self.messages)
                                self.process_client_message(message_from_client, client_with_message)
                        except Exception:
                            logger.info(f'Клиент {client_with_message.getpeername()} '
                                        f'отключился от сервера.')
                            client_with_message.close()
                            self.clients.remove(client_with_message)

                # Если есть сообщения, обрабатываем каждое.
                if send_data_lst and self.messages:
                    for message in self.messages:
                        self.messages.remove(message)
                        for s_listener in send_data_lst:
                            try:
                                if s_listener.getpeername() in self.clients_socket_names \
                                        and s_listener.getpeername() == tuple(message['user']['sock']) \
                                        and (message['action'] == 'presence' or message['action'] == 'get_contacts' or
                                             message['action'] == 'add_contact' or message['action'] == 'del_contact'):
                                    try:
                                        if message['action'] == 'add_contact':
                                            response = self.process_client_message(message, s_listener)
                                            send_message(s_listener, response)
                                            message[
                                                'message_text'] = f'Added to {message[USER][ACCOUNT_NAME]} contact list'
                                            self.database.contact(message[USER][ACCOUNT_NAME], message['contact'],
                                                                  datetime.datetime.now(),
                                                                  message['message_text']
                                                                  )
                                        elif message['action'] == 'del_contact':
                                            response = self.process_client_message(message, s_listener)
                                            # print(response)
                                            send_message(s_listener, response)
                                            message[
                                                'message_text'] = f'Deleted from {message[USER][ACCOUNT_NAME]}' \
                                                                  f' contact list'
                                            self.database.del_contact(message[USER][ACCOUNT_NAME], message['contact'],
                                                                      datetime.datetime.now(),
                                                                      message['message_text'])
                                    except BrokenPipeError:
                                        print('Вах')
                                        send_data_lst.remove(s_listener)
                                    except KeyError:
                                        self.clients.remove(s_listener)
                                        pass
                                elif s_listener.getpeername() in self.clients_socket_names and \
                                        message['action'] == 'message':
                                    try:
                                        response = self.process_client_message(message, s_listener)
                                        print(response)
                                        if len(response['sock_address']) == 0 \
                                                and s_listener.getpeername() == tuple(message['user']['sock']):
                                            response[
                                                'data'] = f'Вы отправили сообщение не существующему либо отключенному '\
                                                          f'адресату {message["to"]}'
                                            send_message(s_listener, response)
                                        elif s_listener.getpeername() == tuple(response['sock_address']):
                                            send_message(s_listener, response)

                                            print(response)
                                            self.database.contact(message[USER][ACCOUNT_NAME], message['to'],
                                                                  datetime.datetime.now(), message['message_text'])
                                    except BrokenPipeError:
                                        print('Вах')
                                        send_data_lst.remove(s_listener)
                                    except IndexError:
                                        pass
                            except OSError:
                                pass



import configparser
import threading
from PyQt6.QtWidgets import QMainWindow, QLabel, QTableView, QMessageBox
from PyQt6.QtGui import QAction
from PyQt6.QtCore import QTimer
from server.config_window import ConfigWindow
from server.add_user import RegisterUser
from server.server_qui import gui_create_model, HistoryWindow, create_stat_model

conflag_lock = threading.Lock()
config = configparser.ConfigParser()


class MainWindowServer(QMainWindow):
    '''Класс - основное окно сервера.'''

    def __init__(self, database, server, config):
        # Конструктор предка
        super().__init__()

        # База данных сервера
        self.database = database

        self.server_thread = server
        self.config = config

        self.messages = QMessageBox()

        # Ярлык выхода
        self.exitAction = QAction('Выход', self)
        self.exitAction.setShortcut('Ctrl+Q')

        # Кнопка обновить список клиентов
        self.refresh_button = QAction('Обновить список', self)

        # Кнопка настроек сервера
        self.config_btn = QAction('Настройки сервера', self)

        # Кнопка регистрации пользователя
        self.register_btn = QAction('Регистрация пользователя', self)

        # Кнопка вывести историю сообщений
        self.show_history_button = QAction('История клиентов', self)

        # Статусбар
        self.statusBar()
        self.statusBar().showMessage('Server Working')

        # Тулбар
        self.toolbar = self.addToolBar('MainBar')
        self.toolbar.addAction(self.exitAction)
        self.toolbar.addAction(self.refresh_button)
        self.toolbar.addAction(self.show_history_button)
        self.toolbar.addAction(self.config_btn)
        self.toolbar.addAction(self.register_btn)

        # Настройки геометрии основного окна
        # Поскольку работать с динамическими размерами мы не умеем, и мало
        # времени на изучение, размер окна фиксирован.
        self.setFixedSize(800, 600)
        self.setWindowTitle('Messaging Server alpha release')

        # Надпись о том, что ниже список подключённых клиентов
        self.label = QLabel('Список подключённых клиентов:', self)
        self.label.setFixedSize(240, 15)
        self.label.move(10, 25)

        # Окно со списком подключённых клиентов.
        self.active_clients_table = QTableView(self)
        self.active_clients_table.move(10, 45)
        self.active_clients_table.setFixedSize(780, 400)

        # Таймер, обновляющий список клиентов 1 раз в секунду
        self.timer = QTimer()
        self.timer.timeout.connect(self.list_update)
        self.timer.start(1000)

        # Связываем кнопки с процедурами
        self.refresh_button.triggered.connect(self.list_update)
        self.show_history_button.triggered.connect(self.show_statistics)
        self.config_btn.triggered.connect(self.server_config)
        self.register_btn.triggered.connect(self.reg_user)

        self.statusBar().showMessage('Server Working')
        self.active_clients_table.setModel(gui_create_model(database))
        self.active_clients_table.resizeColumnsToContents()
        self.active_clients_table.resizeRowsToContents()

    def list_update(self):
        with conflag_lock:
            self.active_clients_table.setModel(
                gui_create_model(self.database))
            self.active_clients_table.resizeColumnsToContents()
            self.active_clients_table.resizeRowsToContents()

    # Функция, создающая окно со статистикой клиентов
    def show_statistics(self):
        global stat_window
        stat_window = HistoryWindow()
        stat_window.history_table.setModel(create_stat_model(self.database))
        stat_window.history_table.resizeColumnsToContents()
        stat_window.history_table.resizeRowsToContents()
        stat_window.show()

    # Функция создающяя окно с настройками сервера.
    def server_config(self):
        global config_window
        # Создаём окно и заносим в него текущие параметры
        config_window = ConfigWindow(self.config)

    def reg_user(self):
        '''Метод создающий окно регистрации пользователя.'''
        global reg_window
        reg_window = RegisterUser(self.database, self.server_thread)
        reg_window.show()

    def new_conn(self):
        self.messages.information(self, 'New connection', ' ')
        self.list_update()

    def make_connection(self, server_obj):
        # print(server_obj.new_connection)
        server_obj.new_connection.connect(self.new_conn)



import socket


class MySocket(socket.socket):
    def __init__(self, *args, **kwargs):
        super(MySocket, self).__init__(*args, **kwargs)
        self.is_sender = bool
        self.is_listener = bool



"""
Опорная схема базы данных:
На стороне сервера БД содержит следующие таблицы:
 - клиент:
    логин;
    информация.
- история_клиента:
    время входа;
    ip-адрес.
- список_контактов (составляется на основании выборки всех записей с id_владельца):
    id_владельца;
    id_клиента.

"""
from sqlalchemy import create_engine, Table, Column, Integer, String, MetaData, ForeignKey, DateTime, Boolean
from sqlalchemy.orm import mapper, sessionmaker
import datetime


class ServerStorage:
    class AllUsers:
        def __init__(self, username, password_hash, ip_address, port, sender_count, recepient_count):
            self.username = username
            self.passwd_hash = password_hash
            self.last_login = datetime.datetime.now()
            self.ip_address = ip_address
            self.port = port
            self.id = None
            self.sender_count = sender_count
            self.recepient_count = recepient_count

    class ClientHistory:
        def __init__(self, user_id, ip_address, port, login_time):
            self.id = None
            self.user_id = user_id
            self.ip_address = ip_address
            self.port = port
            self.login_time = login_time

    class ClientContacts:
        def __init__(self, user_id, contact_name, contact_time, message, send_count, recep_count, is_friend):
            self.user_id = user_id
            self.contact_name = contact_name
            self.contact_time = contact_time
            self.message = message
            self.send_count = send_count
            self.recep_count = recep_count
            self.is_friend = is_friend

    def __init__(self, path):
        self.database_engine = create_engine(
            f'sqlite:///{path}',
            echo=False,
            pool_recycle=7200,
            connect_args={
                'check_same_thread': False})
        self.metadata = MetaData()
        # Создание таблицы пользователей

        users_table = Table('Users', self.metadata,
                            Column('id', Integer, primary_key=True),
                            Column('username', String, unique=True),
                            Column('passwd_hash', String),
                            Column('ip_address', String),
                            Column('port', String),
                            Column('last_login', DateTime),
                            Column('sender_count', Integer),
                            Column('recepient_count', Integer)
                            )
        # Создание таблицы истории активности пользователей
        login_history = Table('login_history', self.metadata,
                              Column('id', Integer, primary_key=True),
                              Column('user_id', ForeignKey('Users.id')),
                              Column('ip_address', String),
                              Column('port', String),
                              Column('login_time', DateTime),
                              )
        # Создание таблицы истории контактов пользователей
        users_contacts = Table('users_contacts', self.metadata,
                               Column('id', Integer, primary_key=True),
                               Column('user_id', ForeignKey('Users.id')),
                               Column('contact_name', String),
                               Column('message', String),
                               Column('contact_time', DateTime),
                               Column('send_count', Integer),
                               Column('recep_count', Integer),
                               Column('is_friend', Boolean)
                               )
        # Создаем таблицы
        self.metadata.create_all(self.database_engine)
        mapper(self.AllUsers, users_table)
        mapper(self.ClientHistory, login_history)
        mapper(self.ClientContacts, users_contacts)
        # Создаем сессию
        session = sessionmaker(bind=self.database_engine)
        self.session = session()

    def user_login(self, username, ip_address, port):
        res = self.session.query(self.AllUsers).filter_by(username=username)
        if res.count():
            user = res.first()
            user.last_login = datetime.datetime.now()
            print(user.ip_address)
            user.ip_address = ip_address
            user.port = port
        else:
            sender_count = 0
            recepient_count = 0
            print(self.get_hash(username))
            user = self.AllUsers(username, self.get_hash(username), ip_address, port, sender_count, recepient_count)
            self.session.add(user)
            self.session.commit()
        date_time = datetime.datetime.now()
        history = self.ClientHistory(user.id, ip_address, port, date_time)
        self.session.add(history)
        self.session.commit()

    def add_user(self, name, passwd_hash):
        """
        Метод регистрации пользователя.
        Принимает имя и хэш пароля, создаёт запись в таблице статистики.
        """
        sender_count = 0
        recepient_count = 0
        user = self.AllUsers(name, passwd_hash, None, None, sender_count, recepient_count)
        self.session.add(user)
        self.session.commit()
        date_time = datetime.datetime.now()
        history = self.ClientHistory(user.id, None, None, date_time)
        self.session.add(history)
        self.session.commit()

    def check_user(self, name):
        """Метод проверяющий существование пользователя."""
        if self.session.query(self.AllUsers).filter_by(username=name).count():
            return True
        else:
            return False

    def get_hash(self, name):
        """Метод получения хэша пароля пользователя."""
        user = self.session.query(self.AllUsers).filter_by(username=name).first()
        return user.passwd_hash

    def contact(self, username, add_contact_name, contact_time, message):
        sender = self.session.query(self.AllUsers).filter_by(username=username).first()

        try:
            recipient = self.session.query(self.AllUsers).filter_by(username=add_contact_name).first()
            sender.sender_count += 1
            recipient.recepient_count += 1
            res = self.session.query(self.AllUsers).filter_by(username=username)
            user = res.first()
            # print(user)
            res = self.session.query(self.ClientContacts).filter_by(user_id=sender.id)
            if res:
                for contact in res:
                    if contact.contact_name == add_contact_name:
                        contact.is_friend = True
                        # print(contact.is_friend)

            is_friend = True
            contacts = self.ClientContacts(user.id, add_contact_name, contact_time, message, sender.sender_count,
                                           recipient.recepient_count, is_friend)
            self.session.add(contacts)
        except AttributeError:
            pass

        self.session.commit()

    def del_contact(self, username, del_contact_name):
        try:
            sender = self.session.query(self.AllUsers).filter_by(username=username).first()
            recipient = self.session.query(self.AllUsers).filter_by(username=del_contact_name).first()
            sender.sender_count += 1
            recipient.recepient_count += 1

            res = self.session.query(self.ClientContacts).filter_by(user_id=sender.id)
            for contact in res:
                if contact.contact_name == del_contact_name:
                    contact.is_friend = False
                    # print(contact.is_friend)
        except AttributeError:
            pass
        self.session.commit()

    def user_list(self, username=None):
        query = self.session.query(
            self.AllUsers.id,
            self.AllUsers.username,
            self.AllUsers.ip_address,
            self.AllUsers.port,
            self.AllUsers.last_login,
            self.AllUsers.sender_count,
            self.AllUsers.recepient_count

        )
        if username:
            query = query.filter(self.AllUsers.username == username)
        return query.all()

    def users_gui(self):
        query = self.session.query(
            self.AllUsers.username,
            self.AllUsers.ip_address,
            self.AllUsers.port,
            self.AllUsers.last_login
        )
        return query.all()

    def stat_gui(self):
        query = self.session.query(
            self.AllUsers.username,
            self.AllUsers.last_login,
            self.AllUsers.sender_count,
            self.AllUsers.recepient_count
        )
        return query.all()

    def history(self, username=None):
        query = self.session.query(
            self.AllUsers.username,
            self.ClientHistory.login_time,
            self.ClientHistory.ip_address,
            self.ClientHistory.port
        ).join(self.AllUsers)
        if username:
            query = query.filter(self.AllUsers.username == username)
        return query.all()

    def contacts_list(self, username):

        query = self.session.query(
            self.AllUsers.username,
            self.ClientContacts.contact_name,
            self.ClientContacts.message,
            self.ClientContacts.contact_time,
            self.ClientContacts.is_friend
        ).join(self.AllUsers)
        if username:
            query = query.filter(self.AllUsers.username == username)
        query_is_friend = query.filter(True == self.ClientContacts.is_friend)
        return query_is_friend.all()

    def to_client_message(self, username):
        query = self.session.query(
            self.AllUsers.username,
            self.ClientContacts.contact_name,
            self.ClientContacts.message,
            self.ClientContacts.contact_time,
            self.ClientContacts.is_friend
        ).join(self.AllUsers)
        if username:
            query = query.filter(self.ClientContacts.contact_name == username)
        return query.all()


if __name__ == '__main__':
    test_db = ServerStorage()
    # print("Версия SQLAlchemy:", sqlalchemy.__version__)
    # test_db.user_login('client_1', '127.0.0.1', 7777)
    # test_db.user_login('client_2', '127.0.0.1', 8888)
    # test_db.user_login('client_3', '127.0.0.1', 7878)
    # test_db.user_login('client_4', '127.0.0.1', 7888)
    # test_db.user_login('client_5', '127.0.0.1', 7888)
    # print('============== test AllUsers ==============')
    # pprint(test_db.user_list())
    # #
    # test_db.contact('client_2', 'client_1', datetime.datetime.now(), 'test_2_1')
    # test_db.contact('client_2', 'client_3', datetime.datetime.now(), 'test_2_3')
    # test_db.contact('client_3', 'client_1', datetime.datetime.now(), 'test_3_1')
    # test_db.contact('client_3', 'client_2', datetime.datetime.now(), 'test_3_2')
    #
    # print('============== test ClientsContacts ==============')
    # test_db.contacts_list('client_2')
    # test_db.contacts_list(None)
    # pprint(test_db.contacts_list('client_2'))
    #
    # print('============== test ClientsHistory ==============')
    # pprint(test_db.history())
    # pprint(test_db.history('client_3'))



"""Новый протокол дескрипторов"""


class NonNegative:

    def __get__(self, instance, owner):
        return instance.__dict__[self.my_attr]

    def __set__(self, instance, value):
        if type(value) is not int:
            raise ValueError("Значение порта может быть только целым числом")
        if value < 0:
            raise ValueError("Не может быть отрицательным")
        instance.__dict__[self.my_attr] = value

    def __set_name__(self, owner, my_attr):
        # owner - владелец атрибута - <class '__main__.Worker'>
        # my_attr - имя атрибута владельца -listen_port
        self.my_attr = my_attr



from setuptools import setup, find_packages

setup(name="baskort_server_messenger",
      version="0.0.2",
      description="messenger_server",
      author="MadEngine",
      author_email="scorpio-84@mail.ru",
      packages=find_packages(),
      install_requires=['PyQt6', 'sqlalchemy']
      )



import os
import sys
import configparser
from PyQt6.QtWidgets import QApplication
from server.main_window import MainWindowServer
from server.server_DB import ServerStorage
from server.my_core import Server
from common.variables import DEFAULT_PORT
from logs.server_log_config import log

logger = log


def main():
    # Загрузка файла конфигурации сервера
    config = configparser.ConfigParser()
    dir_path = os.path.dirname(os.path.realpath(__file__))
    config.read(f"{dir_path}/{'server.ini'}")

    # Инициализация базы данных
    database = ServerStorage(
        os.path.join(
            dir_path,
            'server_database'))

    try:
        if '-p' in sys.argv:
            listen_port = int(sys.argv[sys.argv.index('-p') + 1])
            logger.info(f'PORT : {listen_port}')
            print(sys.argv)
        else:
            listen_port = DEFAULT_PORT
        if 1024 > listen_port > 65535:
            raise ValueError
    except IndexError:
        logger.error('После параметра -\'p\' необходимо указать номер порта.')
        print('После параметра -\'p\' необходимо указать номер порта.')
        sys.exit(1)
    except ValueError:
        logger.error('Номер порт должен находиться в диапазоне  [1024 - 65535]')
        print('Номер порт должен находиться в диапазоне  [1024 - 65535]')
        sys.exit(1)

    # Затем загружаем какой адрес слушать
    try:
        if '-a' in sys.argv:
            listen_address = sys.argv[sys.argv.index('-a') + 1]
        else:
            listen_address = ''
    except IndexError:
        logger.error('После параметра -\'a\' необходимо указать номер порта.')
        print(
            'После параметра \'a\'- необходимо указать адрес')
        sys.exit(1)

    server = Server(listen_address, listen_port, database)
    server.daemon = True
    server.start()

    server_app = QApplication(sys.argv)
    main_window = MainWindowServer(database, server, config)
    main_window.make_connection(server)
    main_window.show()

    # Запускаем GUI
    server_app.exec()


if __name__ == '__main__':
    main()
